		/* A simple server in the internet domain using TCP
   The port number is passed as an argument */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <libgen.h>
#include <signal.h>
#include <limits.h>
#include "LinuxDARwIn.h"
#include "mjpg_streamer.h"

#define INI_FILE_PATH       "../../../Data/config.ini"

#ifdef MX28_1024
#define MOTION_FILE_PATH    "../../../Data/motion_1024.bin"
#else
#define MOTION_FILE_PATH    "../../../Data/motion_4096.bin"
#endif

#define INI_FILE_PATH       "../../../Data/config.ini"
#define SCRIPT_FILE_PATH    "script.asc"

#define U2D_DEV_NAME0       "/dev/ttyUSB0"
#define U2D_DEV_NAME1       "/dev/ttyUSB1"

using namespace Robot;

void error(const char *msg)
{
    perror(msg);
    exit(1);
}

int main(int argc, char *argv[])
{
	int pitchpos, rollpos,r_pos,p_pos; // Initialize the variables for the current dynamixel positions

	printf( "\n===== Master/Slave: Server for DARwIn =====\n\n");
	//////////////////// Framework Initialize ////////////////////////////
	LinuxCM730 linux_cm730("/dev/ttyUSB0");
	CM730 cm730(&linux_cm730);
	if(cm730.Connect() == false)
	{
		printf("Fail to connect CM-730!\n");
		return 0;
	}
	/////////////////////////////////////////////////////////////////////
	minIni* ini = new minIni(INI_FILE_PATH);
    	if(MotionManager::GetInstance()->Initialize(&cm730) == false)
   	{
   	     printf("Initializing Motion Manager failed!\n");
   	     return 0;
	}
  	MotionManager::GetInstance()->LoadINISettings(ini);
	MotionManager::GetInstance()->AddModule((MotionModule*)Action::GetInstance());
	LinuxMotionTimer *motion_timer = new LinuxMotionTimer(MotionManager::GetInstance());
    	motion_timer->Start();

    int firm_ver = 0;
    if(cm730.ReadByte(JointData::ID_HEAD_PAN, MX28::P_VERSION, &firm_ver, 0)  != CM730::SUCCESS)
    {
        fprintf(stderr, "Can't read firmware version from Dynamixel ID %d!! \n\n", JointData::ID_HEAD_PAN);
        exit(0);
    }

    if(0 < firm_ver && firm_ver < 27)
    {
#ifdef MX28_1024
        Action::GetInstance()->LoadFile(MOTION_FILE_PATH);
#else
        fprintf(stderr, "MX-28's firmware is not support 4096 resolution!! \n");
        fprintf(stderr, "Upgrade MX-28's firmware to version 27(0x1B) or higher.\n\n");
        exit(0);
#endif
    }
    else if(27 <= firm_ver)
    {
#ifdef MX28_1024
        fprintf(stderr, "MX-28's firmware is not support 1024 resolution!! \n");
        fprintf(stderr, "Remove '#define MX28_1024' from 'MX28.h' file and rebuild.\n\n");
        exit(0);
#else
        Action::GetInstance()->LoadFile((char*)MOTION_FILE_PATH);
#endif
    }
    else
        exit(0);

    	Action::GetInstance()->m_Joint.SetEnableBody(true, true); // Enables the joints in order to play an action
   	MotionManager::GetInstance()->SetEnable(true);

	Action::GetInstance()->Start(34); // Plays the starting action
	
	while(Action::GetInstance()->IsRunning()) usleep(5*1000);

	Action::GetInstance()->m_Joint.SetEnableBody(false, false); // Disables the joints to allow movement with control input

 	MotionManager::GetInstance()->SetEnable(false);
/*
	cm730.WriteWord(JointData::ID_R_SHOULDER_PITCH, MX28::P_TORQUE_ENABLE, 0, 0);
    	cm730.WriteWord(JointData::ID_R_SHOULDER_ROLL,  MX28::P_TORQUE_ENABLE, 0, 0);
    	cm730.WriteWord(JointData::ID_R_ELBOW,          MX28::P_TORQUE_ENABLE, 0, 0);
*/
    	cm730.WriteByte(JointData::ID_L_SHOULDER_PITCH, MX28::P_P_GAIN, 8, 0);
    	cm730.WriteByte(JointData::ID_L_SHOULDER_ROLL,  MX28::P_P_GAIN, 8, 0);    	
	cm730.WriteByte(JointData::ID_L_ELBOW,          MX28::P_P_GAIN, 8, 0);
	
     int sockfd, newsockfd, portno;
     socklen_t clilen;
     //char buffer[256];
	//int value;
	int disp[4] = {0}; // array used to save the values coming from the client
     struct sockaddr_in serv_addr, cli_addr;
     int n;
     if (argc < 2) {
         fprintf(stderr,"ERROR, no port provided\n");
         exit(1);
     }

     sockfd = socket(AF_INET, SOCK_STREAM, 0);
     if (sockfd < 0)
        error("ERROR opening socket");
     bzero((char *) &serv_addr, sizeof(serv_addr));
     portno = atoi(argv[1]);
     serv_addr.sin_family = AF_INET;
     serv_addr.sin_addr.s_addr = INADDR_ANY;
     serv_addr.sin_port = htons(portno);
     if (bind(sockfd, (struct sockaddr *) &serv_addr,
              sizeof(serv_addr)) < 0)
              error("ERROR on binding");
     listen(sockfd,5);
     clilen = sizeof(cli_addr);
     newsockfd = accept(sockfd,
                 (struct sockaddr *) &cli_addr,
                 &clilen);
     if (newsockfd < 0)

          error("ERROR on accept");

	cm730.ReadWord(JointData::ID_L_SHOULDER_PITCH, MX28::P_PRESENT_POSITION_L, &pitchpos, 0); // Read the current position of dynamixel controlling right arm pitch

	cm730.ReadWord(JointData::ID_L_SHOULDER_ROLL, MX28::P_PRESENT_POSITION_L, &rollpos, 0); // Read the current position of dynamixel controlling right arm roll
 
	while (1) {
     bzero(buffer,256);
     n = read(newsockfd,disp,sizeof(disp)); // Read the array sent from the client and 
     if (n < 0) error("ERROR reading from socket");


/*
	if(cm730.ReadWord(JointData::ID_R_SHOULDER_PITCH, MX28::P_PRESENT_POSITION_L, &value, 0) == CM730::SUCCESS)
	{
		cm730.WriteWord(JointData::ID_R_SHOULDER_PITCH, MX28::P_GOAL_POSITION_L, value + disp[0], 0);
	}
	if(cm730.ReadWord(JointData::ID_R_SHOULDER_ROLL, MX28::P_PRESENT_POSITION_L, &value, 0) == CM730::SUCCESS)
	{
		cm730.WriteWord(JointData::ID_R_SHOULDER_ROLL, MX28::P_GOAL_POSITION_L, value + disp[1], 0);
	}
	if(cm730.ReadWord(JointData::ID_R_ELBOW, MX28::P_PRESENT_POSITION_L, &value, 0) == CM730::SUCCESS)
	{
		cm730.WriteWord(JointData::ID_R_ELBOW, MX28::P_GOAL_POSITION_L, value + disp[2], 0);
	}


*/

	r_pos = rollpos+11.378*disp[0]; // converts the value from wiimote to dynamixel rotation and adds it to the roll present position
	

	p_pos = pitchpos+11.378*disp[1]; // converts the value from wiimote to dynamixel rotation and adds it to the pitch present position


		if (rollpos>=2428 || rollpos<=1200) // Establish a limit so the robot will not hit itself

			{
				printf("Limit has been reached"); 

			}
		else 
			{
				cm730.WriteWord(JointData::ID_L_SHOULDER_ROLL, MX28::P_GOAL_POSITION_L, r_pos, 0);
			}

	
		if (pitchpos<0 || pitchpos>3038)
			{
				printf("Limit has been reached");
			}
		else
			{
				cm730.WriteWord(JointData::ID_L_SHOULDER_PITCH, MX28::P_GOAL_POSITION_L, p_pos, 0);
			}






		//cm730.WriteWord(JointData::ID_R_ELBOW, MX28::P_GOAL_POSITION_L, disp[2], 0);


	//for(int i = 0; i < 3; i++)
	//{	
 	    //printf("ID[%d]: %d\n",i+25,disp[i]);
	//}
	
	printf("x:%d\n", disp[0]); // print the output values
	printf("y:%d\n", disp[1]);
	printf("z:%d\n", disp[2]);
	printf("Reset:%d\n",disp[3]);


     n = write(newsockfd,"0",1);
     if (n < 0) error("ERROR writing to socket");
	}
     close(newsockfd);
     close(sockfd);
     return 0;
}
